unit ZipInter;   {Interface to DLL info-unz.dll} 

interface

uses WinTypes,WinProcs;

type TDirtype=array[0..79] of char;
     TZipRec=record
       internal:array[0..7] of byte;  {Used internally by the dll}
       Time,                     {file time}
       Size,                     {file size}
       CompressSize,             {size in zipfile}
       headeroffset: Longint;    {file offset in zip: needed in unzipfile}
       FileName: tdirtype;       {file name}
       PackMethod: word;         {see below}
       Attr: Byte;               {file attribute}
     end;

{PackMethod:      supported by unzipfile
  0:   stored             x
  1:   shrunk
  2..5:reduced
  6:   imploded           x
  7:   tokenized
  8:   deflated           x
}

const zip_ok=0;
      zip_FileError=-1;        {Error reading zip file}
      zip_InternalError=-2;    {Error in zip file format}
      zip_NoMoreItems=1;       {Everything read}


const   {Error codes, delivered by unzipfile}
  unzip_Ok=0;               {Unpacked ok}
  unzip_CRCErr=1;           {CRC error}
  unzip_WriteErr=2;         {Error writing out file: maybe disk full} 
  unzip_ReadErr=3;          {Error reading zip file}
  unzip_ZipFileErr=4;       {Error in zip structure}  
  unzip_UserAbort=5;        {Aborted by user}
  unzip_NotSupported=6;     {ZIP Method not supported!}
  unzip_Encrypted=7;        {Zipfile encrypted}
  unzip_InUse=-1;           {DLL in use by other program!}

{**************************** DLL Version ****************************}

function GetUnzipDllVersion:word;
{Hi byte=number before period, Lo byte=number after period} 
{Later versions will be downward compatible}

{******************** ZIP central directory access *******************}

{The following 3 functions can be called in a loop to retreive all
 the files in the given zip file. 
 Use these functions similar to findfirst and findnext:

 Example:
 var r:tziprec;

 rc:=GetFirstInZip(zipname,r);
 while rc=zip_ok do
   DosomethingWithData(r);
   rc:=GetNextInZip(r);
 end;
 closezipfile(r);
 case rc of
   zip_FileError:messagebox(hwindow,'Error reading ZIP file!',zipname,mb_ok);
   zip_InternalError:messagebox(hwindow,'Internal error in ZIP file!',zipname,mb_ok);
 end;
}

function GetFirstInZip(zipfilename:pchar;var zprec:tZipRec):integer;
{zipfilename: filename of zip file}
{zprec:       record, will be filled with zipfile data}

function GetNextInZip(var Zprec:tZiprec):integer;
{zprec:       record, will be filled with zipfile data,
 do not change the 'internal' field received from previous calls!}

procedure CloseZipFile(var Zprec:tZiprec);
{Call after last GetNextInZip call to free buffer}

{********************* Test if file is a ZIP file ********************}

function isZip(filename:pchar):boolean;
{Tests if given file is a zip file (only test for PK#3#4 at the beginning)}

{***************** Get Unzip Methods supported by DLL ****************}
{Currently (version 1.0) these are stored (0), shrunk (1),
                            imploded (6) and deflated (8)}

function GetSupportedMethods:longint;
{Method 0 supported -> bit 0 = 1,
 Method 8 supported -> bit 8 = 1,
 etc.}

{********************* unzip a file from ZIP-file ********************}

function unzipfile(in_name:pchar;out_name:pchar;offset:longint;
  hFileAction:hwnd;cm_index:integer):integer; far;
{usage:
 in_name:      name of zip file with full path
 out_name:     desired name for out file
 offset:       header position of desired file in zipfile, found in tZiprec
 hFileAction:  handle to dialog box showing advance of decompression (optional),
               or zero when only keyboard shall be checked
 cm_index:     - if hfileaction<>0 : notification code sent in a wm_command
                 message to the dialog to update percent-bar
               - if hfileaction=0  : virtual key code of key the user must press
                 to interrupt unzipping, i.e. vk_escape
                  
 Return value: one of the above unzip_xxx codes

 Example for handling the cm_index message in a progress dialog:

 unzipfile(......,cm_showpercent);

 ...

 procedure TFileActionDialog.wmcommand(var msg:tmessage);
 var ppercent:^word;
 begin
   TDialog.WMCommand(msg);
   if msg.wparam=cm_showpercent then begin
     ppercent:=pointer(lparam);
     if ppercent<>nil then begin
       if (ppercent^>=0) and (ppercent^<=100) then
         SetProgressBar(ppercent^);
       if UserPressedAbort then
         ppercent^:=$ffff
       else
         ppercent^:=0;
       end;
     end;
   end;
 end;
}

{*********************************************************************}

implementation

function GetUnzipDllVersion; external 'INFO-UNZ' index 1;
function GetFirstInZip; external 'INFO-UNZ' index 2;
function GetNextInZip; external 'INFO-UNZ' index 3;
procedure CloseZipFile; external 'INFO-UNZ' index 4;
function isZip; external 'INFO-UNZ' index 5;
function GetSupportedMethods; external 'INFO-UNZ' index 6;
function unzipfile; external 'INFO-UNZ' index 7;

end.
